@@ -28,8 +28,6 @@
 #include "xalloc.h"
 #include "xerror.h"
 #include "format-invalid.h"
-#include "error.h"
-#include "error-progname.h"
 #include "gettext.h"
 
 #define _(str) gettext (str)
@@ -249,8 +247,9 @@ format_get_number_of_directives (void *descr)
 }
 
 static bool
-format_check (const lex_pos_ty *pos, void *msgid_descr, void *msgstr_descr,
-	      bool equality, bool noisy, const char *pretty_msgstr)
+format_check (void *msgid_descr, void *msgstr_descr, bool equality,
+	      formatstring_error_logger_t error_logger,
+	      const char *pretty_msgstr)
 {
   struct spec *spec1 = (struct spec *) msgid_descr;
   struct spec *spec2 = (struct spec *) msgstr_descr;
@@ -262,28 +261,18 @@ format_check (const lex_pos_ty *pos, void *msgid_descr, void *msgstr_descr,
       ? spec1->unnumbered_arg_count != spec2->unnumbered_arg_count
       : spec1->unnumbered_arg_count < spec2->unnumbered_arg_count)
     {
-      if (noisy)
-	{
-	  error_with_progname = false;
-	  error_at_line (0, 0, pos->file_name, pos->line_number,
-			 _("number of format specifications in 'msgid' and '%s' does not match"),
-			 pretty_msgstr);
-	  error_with_progname = true;
-	}
+      if (error_logger)
+	error_logger (_("number of format specifications in 'msgid' and '%s' does not match"),
+		      pretty_msgstr);
       err = true;
     }
   else
     for (i = 0; i < spec2->unnumbered_arg_count; i++)
       if (spec1->unnumbered[i].type != spec2->unnumbered[i].type)
 	{
-	  if (noisy)
-	    {
-	      error_with_progname = false;
-	      error_at_line (0, 0, pos->file_name, pos->line_number,
-			     _("format specifications in 'msgid' and '%s' for argument %u are not the same"),
-			     pretty_msgstr, i + 1);
-	      error_with_progname = true;
-	    }
+	  if (error_logger)
+	    error_logger (_("format specifications in 'msgid' and '%s' for argument %u are not the same"),
+			  pretty_msgstr, i + 1);
 	  err = true;
 	}
 
