@@ -31,8 +31,6 @@
 #include "xallocsa.h"
 #include "xerror.h"
 #include "format-invalid.h"
-#include "error.h"
-#include "error-progname.h"
 #include "gettext.h"
 
 #define _(str) gettext (str)
@@ -165,6 +163,7 @@ message_format_parse (const char *format, struct spec *spec,
 	  const char *element_start;
 	  const char *element_end;
 	  size_t n;
+	  char *element_alloced;
 	  char *element;
 	  unsigned int number;
 	  enum format_arg_type type;
@@ -194,7 +193,7 @@ message_format_parse (const char *format, struct spec *spec,
 	  element_end = format++;
 
 	  n = element_end - element_start;
-	  element = (char *) xallocsa (n + 1);
+	  element = element_alloced = (char *) xallocsa (n + 1);
 	  memcpy (element, element_start, n);
 	  element[n] = '\0';
 
@@ -202,7 +201,7 @@ message_format_parse (const char *format, struct spec *spec,
 	    {
 	      *invalid_reason =
 		xasprintf (_("In the directive number %u, '{' is not followed by an argument number."), spec->directives);
-	      freesa (element);
+	      freesa (element_alloced);
 	      return false;
 	    }
 	  number = 0;
@@ -236,7 +235,7 @@ message_format_parse (const char *format, struct spec *spec,
 		    {
 		      *invalid_reason =
 			xasprintf (_("In the directive number %u, the substring \"%s\" is not a valid date/time style."), spec->directives, element);
-		      freesa (element);
+		      freesa (element_alloced);
 		      return false;
 		    }
 		}
@@ -246,7 +245,7 @@ message_format_parse (const char *format, struct spec *spec,
 		  element -= 4;
 		  *invalid_reason =
 		    xasprintf (_("In the directive number %u, \"%s\" is not followed by a comma."), spec->directives, element);
-		  freesa (element);
+		  freesa (element_alloced);
 		  return false;
 		}
 	    }
@@ -268,7 +267,7 @@ message_format_parse (const char *format, struct spec *spec,
 		    {
 		      *invalid_reason =
 			xasprintf (_("In the directive number %u, the substring \"%s\" is not a valid number style."), spec->directives, element);
-		      freesa (element);
+		      freesa (element_alloced);
 		      return false;
 		    }
 		}
@@ -278,7 +277,7 @@ message_format_parse (const char *format, struct spec *spec,
 		  element -= 6;
 		  *invalid_reason =
 		    xasprintf (_("In the directive number %u, \"%s\" is not followed by a comma."), spec->directives, element);
-		  freesa (element);
+		  freesa (element_alloced);
 		  return false;
 		}
 	    }
@@ -295,7 +294,7 @@ message_format_parse (const char *format, struct spec *spec,
 		    ;
 		  else
 		    {
-		      freesa (element);
+		      freesa (element_alloced);
 		      return false;
 		    }
 		}
@@ -305,7 +304,7 @@ message_format_parse (const char *format, struct spec *spec,
 		  element -= 6;
 		  *invalid_reason =
 		    xasprintf (_("In the directive number %u, \"%s\" is not followed by a comma."), spec->directives, element);
-		  freesa (element);
+		  freesa (element_alloced);
 		  return false;
 		}
 	    }
@@ -313,10 +312,10 @@ message_format_parse (const char *format, struct spec *spec,
 	    {
 	      *invalid_reason =
 		xasprintf (_("In the directive number %u, the argument number is not followed by a comma and one of \"%s\", \"%s\", \"%s\", \"%s\"."), spec->directives, "time", "date", "number", "choice");
-	      freesa (element);
+	      freesa (element_alloced);
 	      return false;
 	    }
-	  freesa (element);
+	  freesa (element_alloced);
 
 	  if (spec->allocated == spec->numbered_arg_count)
 	    {
@@ -700,8 +699,9 @@ format_get_number_of_directives (void *descr)
 }
 
 static bool
-format_check (const lex_pos_ty *pos, void *msgid_descr, void *msgstr_descr,
-	      bool equality, bool noisy, const char *pretty_msgstr)
+format_check (void *msgid_descr, void *msgstr_descr, bool equality,
+	      formatstring_error_logger_t error_logger,
+	      const char *pretty_msgstr)
 {
   struct spec *spec1 = (struct spec *) msgid_descr;
   struct spec *spec2 = (struct spec *) msgstr_descr;
@@ -725,14 +725,9 @@ format_check (const lex_pos_ty *pos, void *msgid_descr, void *msgstr_descr,
 
 	  if (cmp > 0)
 	    {
-	      if (noisy)
-		{
-		  error_with_progname = false;
-		  error_at_line (0, 0, pos->file_name, pos->line_number,
-				 _("a format specification for argument {%u}, as in '%s', doesn't exist in 'msgid'"),
-				 spec2->numbered[j].number, pretty_msgstr);
-		  error_with_progname = true;
-		}
+	      if (error_logger)
+		error_logger (_("a format specification for argument {%u}, as in '%s', doesn't exist in 'msgid'"),
+			      spec2->numbered[j].number, pretty_msgstr);
 	      err = true;
 	      break;
 	    }
@@ -740,14 +735,9 @@ format_check (const lex_pos_ty *pos, void *msgid_descr, void *msgstr_descr,
 	    {
 	      if (equality)
 		{
-		  if (noisy)
-		    {
-		      error_with_progname = false;
-		      error_at_line (0, 0, pos->file_name, pos->line_number,
-				     _("a format specification for argument {%u} doesn't exist in '%s'"),
-				     spec1->numbered[i].number, pretty_msgstr);
-		      error_with_progname = true;
-		    }
+		  if (error_logger)
+		    error_logger (_("a format specification for argument {%u} doesn't exist in '%s'"),
+				  spec1->numbered[i].number, pretty_msgstr);
 		  err = true;
 		  break;
 		}
@@ -765,15 +755,9 @@ format_check (const lex_pos_ty *pos, void *msgid_descr, void *msgstr_descr,
 	      {
 		if (spec1->numbered[i].type != spec2->numbered[j].type)
 		  {
-		    if (noisy)
-		      {
-			error_with_progname = false;
-			error_at_line (0, 0, pos->file_name, pos->line_number,
-				       _("format specifications in 'msgid' and '%s' for argument {%u} are not the same"),
-				       pretty_msgstr,
-				       spec2->numbered[j].number);
-			error_with_progname = true;
-		      }
+		    if (error_logger)
+		      error_logger (_("format specifications in 'msgid' and '%s' for argument {%u} are not the same"),
+				    pretty_msgstr, spec2->numbered[j].number);
 		    err = true;
 		    break;
 		  }
