@@ -1,5 +1,5 @@
 /* Python format strings.
-   Copyright (C) 2001-2003 Free Software Foundation, Inc.
+   Copyright (C) 2001-2004 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software; you can redistribute it and/or modify
@@ -29,8 +29,6 @@
 #include "xalloc.h"
 #include "xerror.h"
 #include "format-invalid.h"
-#include "error.h"
-#include "error-progname.h"
 #include "gettext.h"
 
 #define _(str) gettext (str)
@@ -115,7 +113,7 @@ named_arg_compare (const void *p1, const void *p2)
   xstrdup (_("The string refers to arguments both through argument names and through unnamed argument specifications."))
 
 static void *
-format_parse (const char *format, char **invalid_reason)
+format_parse (const char *format, bool translated, char **invalid_reason)
 {
   struct spec spec;
   struct spec *result;
@@ -391,8 +389,9 @@ format_get_number_of_directives (void *descr)
 }
 
 static bool
-format_check (const lex_pos_ty *pos, void *msgid_descr, void *msgstr_descr,
-	      bool equality, bool noisy, const char *pretty_msgstr)
+format_check (void *msgid_descr, void *msgstr_descr, bool equality,
+	      formatstring_error_logger_t error_logger,
+	      const char *pretty_msgstr)
 {
   struct spec *spec1 = (struct spec *) msgid_descr;
   struct spec *spec2 = (struct spec *) msgstr_descr;
@@ -400,26 +399,16 @@ format_check (const lex_pos_ty *pos, void *msgid_descr, void *msgstr_descr,
 
   if (spec1->named_arg_count > 0 && spec2->unnamed_arg_count > 0)
     {
-      if (noisy)
-	{
-	  error_with_progname = false;
-	  error_at_line (0, 0, pos->file_name, pos->line_number,
-			 _("format specifications in 'msgid' expect a mapping, those in '%s' expect a tuple"),
-			 pretty_msgstr);
-	  error_with_progname = true;
-	}
+      if (error_logger)
+	error_logger (_("format specifications in 'msgid' expect a mapping, those in '%s' expect a tuple"),
+		      pretty_msgstr);
       err = true;
     }
   else if (spec1->unnamed_arg_count > 0 && spec2->named_arg_count > 0)
     {
-      if (noisy)
-	{
-	  error_with_progname = false;
-	  error_at_line (0, 0, pos->file_name, pos->line_number,
-			 _("format specifications in 'msgid' expect a tuple, those in '%s' expect a mapping"),
-			 pretty_msgstr);
-	  error_with_progname = true;
-	}
+      if (error_logger)
+	error_logger (_("format specifications in 'msgid' expect a tuple, those in '%s' expect a mapping"),
+		      pretty_msgstr);
       err = true;
     }
   else
@@ -440,14 +429,9 @@ format_check (const lex_pos_ty *pos, void *msgid_descr, void *msgstr_descr,
 
 	      if (cmp > 0)
 		{
-		  if (noisy)
-		    {
-		      error_with_progname = false;
-		      error_at_line (0, 0, pos->file_name, pos->line_number,
-				     _("a format specification for argument '%s', as in '%s', doesn't exist in 'msgid'"),
-				     spec2->named[j].name, pretty_msgstr);
-		      error_with_progname = true;
-		    }
+		  if (error_logger)
+		    error_logger (_("a format specification for argument '%s', as in '%s', doesn't exist in 'msgid'"),
+				  spec2->named[j].name, pretty_msgstr);
 		  err = true;
 		  break;
 		}
@@ -455,14 +439,9 @@ format_check (const lex_pos_ty *pos, void *msgid_descr, void *msgstr_descr,
 		{
 		  if (equality)
 		    {
-		      if (noisy)
-			{
-			  error_with_progname = false;
-			  error_at_line (0, 0, pos->file_name, pos->line_number,
-					 _("a format specification for argument '%s' doesn't exist in '%s'"),
-					 spec1->named[i].name, pretty_msgstr);
-			  error_with_progname = true;
-			}
+		      if (error_logger)
+			error_logger (_("a format specification for argument '%s' doesn't exist in '%s'"),
+				      spec1->named[i].name, pretty_msgstr);
 		      err = true;
 		      break;
 		    }
@@ -480,16 +459,9 @@ format_check (const lex_pos_ty *pos, void *msgid_descr, void *msgstr_descr,
 		  {
 		    if (spec1->named[i].type != spec2->named[j].type)
 		      {
-			if (noisy)
-			  {
-			    error_with_progname = false;
-			    error_at_line (0, 0, pos->file_name,
-					   pos->line_number,
-					   _("format specifications in 'msgid' and '%s' for argument '%s' are not the same"),
-					   pretty_msgstr,
-					   spec2->named[j].name);
-			    error_with_progname = true;
-			  }
+			if (error_logger)
+			  error_logger (_("format specifications in 'msgid' and '%s' for argument '%s' are not the same"),
+					pretty_msgstr, spec2->named[j].name);
 			err = true;
 			break;
 		      }
@@ -509,28 +481,18 @@ format_check (const lex_pos_ty *pos, void *msgid_descr, void *msgstr_descr,
 	      ? spec1->unnamed_arg_count != spec2->unnamed_arg_count
 	      : spec1->unnamed_arg_count < spec2->unnamed_arg_count)
 	    {
-	      if (noisy)
-		{
-		  error_with_progname = false;
-		  error_at_line (0, 0, pos->file_name, pos->line_number,
-				 _("number of format specifications in 'msgid' and '%s' does not match"),
-				 pretty_msgstr);
-		  error_with_progname = true;
-		}
+	      if (error_logger)
+		error_logger (_("number of format specifications in 'msgid' and '%s' does not match"),
+			      pretty_msgstr);
 	      err = true;
 	    }
 	  else
 	    for (i = 0; i < spec2->unnamed_arg_count; i++)
 	      if (spec1->unnamed[i].type != spec2->unnamed[i].type)
 		{
-		  if (noisy)
-		    {
-		      error_with_progname = false;
-		      error_at_line (0, 0, pos->file_name, pos->line_number,
-				     _("format specifications in 'msgid' and '%s' for argument %u are not the same"),
-				     pretty_msgstr, i + 1);
-		      error_with_progname = true;
-		    }
+		  if (error_logger)
+		    error_logger (_("format specifications in 'msgid' and '%s' for argument %u are not the same"),
+				  pretty_msgstr, i + 1);
 		  err = true;
 		}
 	}
@@ -653,7 +615,7 @@ main ()
 	line[--line_len] = '\0';
 
       invalid_reason = NULL;
-      descr = format_parse (line, &invalid_reason);
+      descr = format_parse (line, false, &invalid_reason);
 
       format_print (descr);
       printf ("\n");
