@@ -1,5 +1,5 @@
 /* YCP and Smalltalk format strings.
-   Copyright (C) 2001-2003 Free Software Foundation, Inc.
+   Copyright (C) 2001-2004 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software; you can redistribute it and/or modify
@@ -28,8 +28,6 @@
 #include "xalloc.h"
 #include "xerror.h"
 #include "format-invalid.h"
-#include "error.h"
-#include "error-progname.h"
 #include "gettext.h"
 
 #define _(str) gettext (str)
@@ -50,7 +48,7 @@ struct spec
 
 
 static void *
-format_parse (const char *format, char **invalid_reason)
+format_parse (const char *format, bool translated, char **invalid_reason)
 {
   struct spec spec;
   struct spec *result;
@@ -113,8 +111,9 @@ format_get_number_of_directives (void *descr)
 }
 
 static bool
-format_check (const lex_pos_ty *pos, void *msgid_descr, void *msgstr_descr,
-	      bool equality, bool noisy, const char *pretty_msgstr)
+format_check (void *msgid_descr, void *msgstr_descr, bool equality,
+	      formatstring_error_logger_t error_logger,
+	      const char *pretty_msgstr)
 {
   struct spec *spec1 = (struct spec *) msgid_descr;
   struct spec *spec2 = (struct spec *) msgstr_descr;
@@ -128,16 +127,11 @@ format_check (const lex_pos_ty *pos, void *msgid_descr, void *msgstr_descr,
 
       if (equality ? (arg_used1 != arg_used2) : (!arg_used1 && arg_used2))
 	{
-	  if (noisy)
-	    {
-	      error_with_progname = false;
-	      error_at_line (0, 0, pos->file_name, pos->line_number,
-			     arg_used1
-			     ? _("a format specification for argument %u doesn't exist in '%s'")
-			     : _("a format specification for argument %u, as in '%s', doesn't exist in 'msgid'"),
-			     i + 1, pretty_msgstr);
-	      error_with_progname = true;
-	    }
+	  if (error_logger)
+	    error_logger (arg_used1
+			  ? _("a format specification for argument %u doesn't exist in '%s'")
+			  : _("a format specification for argument %u, as in '%s', doesn't exist in 'msgid'"),
+			  i + 1, pretty_msgstr);
 	  err = true;
 	  break;
 	}
@@ -216,7 +210,7 @@ main ()
 	line[--line_len] = '\0';
 
       invalid_reason = NULL;
-      descr = format_parse (line, &invalid_reason);
+      descr = format_parse (line, false, &invalid_reason);
 
       format_print (descr);
       printf ("\n");
