@@ -1,5 +1,5 @@
 /* Converts Uniforum style .po files to binary .mo files
-   Copyright (C) 1995-1998, 2000-2003 Free Software Foundation, Inc.
+   Copyright (C) 1995-1998, 2000-2005 Free Software Foundation, Inc.
    Written by Ulrich Drepper <drepper@gnu.ai.mit.edu>, April 1995.
 
    This program is free software; you can redistribute it and/or modify
@@ -28,6 +28,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <stdarg.h>
 #include <locale.h>
 
 #include "closeout.h"
@@ -48,6 +49,8 @@
 #include "msgfmt.h"
 #include "write-mo.h"
 #include "write-java.h"
+#include "write-csharp.h"
+#include "write-resources.h"
 #include "write-tcl.h"
 #include "write-qt.h"
 
@@ -91,6 +94,15 @@ static const char *java_resource_name;
 static const char *java_locale_name;
 static const char *java_class_directory;
 
+/* C# mode output file specification.  */
+static bool csharp_mode;
+static const char *csharp_resource_name;
+static const char *csharp_locale_name;
+static const char *csharp_base_directory;
+
+/* C# resources mode output file specification.  */
+static bool csharp_resources_mode;
+
 /* Tcl mode output file specification.  */
 static bool tcl_mode;
 static const char *tcl_locale_name;
@@ -160,6 +172,8 @@ static const struct option long_options[] =
   { "check-domain", no_argument, NULL, CHAR_MAX + 2 },
   { "check-format", no_argument, NULL, CHAR_MAX + 3 },
   { "check-header", no_argument, NULL, CHAR_MAX + 4 },
+  { "csharp", no_argument, NULL, CHAR_MAX + 10 },
+  { "csharp-resources", no_argument, NULL, CHAR_MAX + 11 },
   { "directory", required_argument, NULL, 'D' },
   { "help", no_argument, NULL, 'h' },
   { "java", no_argument, NULL, 'j' },
@@ -251,6 +265,7 @@ main (int argc, char *argv[])
 	break;
       case 'd':
 	java_class_directory = optarg;
+	csharp_base_directory = optarg;
 	tcl_base_directory = optarg;
 	break;
       case 'D':
@@ -267,6 +282,7 @@ main (int argc, char *argv[])
 	break;
       case 'l':
 	java_locale_name = optarg;
+	csharp_locale_name = optarg;
 	tcl_locale_name = optarg;
 	break;
       case 'o':
@@ -277,6 +293,7 @@ main (int argc, char *argv[])
 	break;
       case 'r':
 	java_resource_name = optarg;
+	csharp_resource_name = optarg;
 	break;
       case 'S':
 	strict_uniforum = true;
@@ -325,6 +342,12 @@ main (int argc, char *argv[])
       case CHAR_MAX + 9: /* --qt */
 	qt_mode = true;
 	break;
+      case CHAR_MAX + 10: /* --csharp */
+	csharp_mode = true;
+	break;
+      case CHAR_MAX + 11: /* --csharp-resources */
+	csharp_resources_mode = true;
+	break;
       default:
 	usage (EXIT_FAILURE);
 	break;
@@ -339,7 +362,7 @@ main (int argc, char *argv[])
 This is free software; see the source for copying conditions.  There is NO\n\
 warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
 "),
-	      "1995-1998, 2000-2003");
+	      "1995-1998, 2000-2005");
       printf (_("Written by %s.\n"), "Ulrich Drepper");
       exit (EXIT_SUCCESS);
     }
@@ -356,15 +379,33 @@ warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
     }
 
   /* Check for contradicting options.  */
-  if (java_mode && tcl_mode)
-    error (EXIT_FAILURE, 0, _("%s and %s are mutually exclusive"),
-	   "--java", "--tcl");
-  if (java_mode && qt_mode)
-    error (EXIT_FAILURE, 0, _("%s and %s are mutually exclusive"),
-	   "--java", "--qt");
-  if (tcl_mode && qt_mode)
-    error (EXIT_FAILURE, 0, _("%s and %s are mutually exclusive"),
-	   "--tcl", "--qt");
+  {
+    unsigned int modes =
+      (java_mode ? 1 : 0)
+      | (csharp_mode ? 2 : 0)
+      | (csharp_resources_mode ? 4 : 0)
+      | (tcl_mode ? 8 : 0)
+      | (qt_mode ? 16 : 0);
+    static const char *mode_options[] =
+      { "--java", "--csharp", "--csharp-resources", "--tcl", "--qt" };
+    /* More than one bit set?  */
+    if (modes & (modes - 1))
+      {
+	const char *first_option;
+	const char *second_option;
+	unsigned int i;
+	for (i = 0; ; i++)
+	  if (modes & (1 << i))
+	    break;
+	first_option = mode_options[i];
+	for (i = i + 1; ; i++)
+	  if (modes & (1 << i))
+	    break;
+	second_option = mode_options[i];
+	error (EXIT_FAILURE, 0, _("%s and %s are mutually exclusive"),
+	       first_option, second_option);
+      }
+  }
   if (java_mode)
     {
       if (output_file_name != NULL)
@@ -380,6 +421,28 @@ warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
 	  usage (EXIT_FAILURE);
 	}
     }
+  else if (csharp_mode)
+    {
+      if (output_file_name != NULL)
+	{
+	  error (EXIT_FAILURE, 0, _("%s and %s are mutually exclusive"),
+		 "--csharp", "--output-file");
+	}
+      if (csharp_locale_name == NULL)
+	{
+	  error (EXIT_SUCCESS, 0,
+		 _("%s requires a \"-l locale\" specification"),
+		 "--csharp");
+	  usage (EXIT_FAILURE);
+	}
+      if (csharp_base_directory == NULL)
+	{
+	  error (EXIT_SUCCESS, 0,
+		 _("%s requires a \"-d directory\" specification"),
+		 "--csharp");
+	  usage (EXIT_FAILURE);
+	}
+    }
   else if (tcl_mode)
     {
       if (output_file_name != NULL)
@@ -406,20 +469,20 @@ warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
     {
       if (java_resource_name != NULL)
 	{
-	  error (EXIT_SUCCESS, 0, _("%s is only valid with %s"),
-		 "--resource", "--java");
+	  error (EXIT_SUCCESS, 0, _("%s is only valid with %s or %s"),
+		 "--resource", "--java", "--csharp");
 	  usage (EXIT_FAILURE);
 	}
       if (java_locale_name != NULL)
 	{
-	  error (EXIT_SUCCESS, 0, _("%s is only valid with %s or %s"),
-		 "--locale", "--java", "--tcl");
+	  error (EXIT_SUCCESS, 0, _("%s is only valid with %s, %s or %s"),
+		 "--locale", "--java", "--csharp", "--tcl");
 	  usage (EXIT_FAILURE);
 	}
       if (java_class_directory != NULL)
 	{
-	  error (EXIT_SUCCESS, 0, _("%s is only valid with %s or %s"),
-		 "-d", "--java", "--tcl");
+	  error (EXIT_SUCCESS, 0, _("%s is only valid with %s, %s or %s"),
+		 "-d", "--java", "--csharp", "--tcl");
 	  usage (EXIT_FAILURE);
 	}
     }
@@ -429,8 +492,9 @@ warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
   if (output_file_name != NULL)
     current_domain =
       new_domain (output_file_name,
-		  !qt_mode && strict_uniforum ? add_mo_suffix (output_file_name)
-					      : output_file_name);
+		  strict_uniforum && !csharp_resources_mode && !qt_mode
+		  ? add_mo_suffix (output_file_name)
+		  : output_file_name);
 
   /* Process all given .po files.  */
   while (argc > optind)
@@ -473,6 +537,20 @@ warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\
 				    java_class_directory, assume_java2))
 	    exit_status = EXIT_FAILURE;
 	}
+      else if (csharp_mode)
+	{
+	  if (msgdomain_write_csharp (domain->mlp, canon_encoding,
+				      csharp_resource_name, csharp_locale_name,
+				      csharp_base_directory))
+	    exit_status = EXIT_FAILURE;
+	}
+      else if (csharp_resources_mode)
+	{
+	  if (msgdomain_write_csharp_resources (domain->mlp, canon_encoding,
+						domain->domain_name,
+						domain->file_name))
+	    exit_status = EXIT_FAILURE;
+	}
       else if (tcl_mode)
 	{
 	  if (msgdomain_write_tcl (domain->mlp, canon_encoding,
@@ -560,6 +638,10 @@ Operation mode:\n"));
       printf (_("\
       --java2                 like --java, and assume Java2 (JDK 1.2 or higher)\n"));
       printf (_("\
+      --csharp                C# mode: generate a .NET .dll file\n"));
+      printf (_("\
+      --csharp-resources      C# resources mode: generate a .NET .resources file\n"));
+      printf (_("\
       --tcl                   Tcl mode: generate a tcl/msgcat .msg file\n"));
       printf (_("\
       --qt                    Qt mode: generate a Qt .qm file\n"));
@@ -588,6 +670,18 @@ written under the specified directory.\n\
 "));
       printf ("\n");
       printf (_("\
+Output file location in C# mode:\n"));
+      printf (_("\
+  -r, --resource=RESOURCE     resource name\n"));
+      printf (_("\
+  -l, --locale=LOCALE         locale name, either language or language_COUNTRY\n"));
+      printf (_("\
+  -d DIRECTORY                base directory for locale dependent .dll files\n"));
+      printf (_("\
+The -l and -d options are mandatory.  The .dll file is written in a\n\
+subdirectory of the specified directory whose name depends on the locale.\n"));
+      printf ("\n");
+      printf (_("\
 Output file location in Tcl mode:\n"));
       printf (_("\
   -l, --locale=LOCALE         locale name, either language or language_COUNTRY\n"));
@@ -1049,6 +1143,25 @@ check_plural (message_list_ty *mlp)
 }
 
 
+/* Signal an error when checking format strings.  */
+static lex_pos_ty curr_msgid_pos;
+static void
+formatstring_error_logger (const char *format, ...)
+{
+  va_list args;
+
+  va_start (args, format);
+  fprintf (stderr, "%s:%lu: ",
+	   curr_msgid_pos.file_name,
+	   (unsigned long) curr_msgid_pos.line_number);
+  vfprintf (stderr, format, args);
+  putc ('\n', stderr);
+  fflush (stderr);
+  va_end (args);
+  ++error_message_count;
+}
+
+
 /* Perform miscellaneous checks on a message.  */
 static void
 check_pair (const char *msgid,
@@ -1058,7 +1171,6 @@ check_pair (const char *msgid,
 	    const lex_pos_ty *msgstr_pos, enum is_format is_format[NFORMATS])
 {
   int has_newline;
-  size_t i;
   unsigned int j;
   const char *p;
 
@@ -1156,81 +1268,13 @@ check_pair (const char *msgid,
 
   if (check_format_strings)
     /* Test 3: Check whether both formats strings contain the same number
-       of format specifications.
-       We check only those messages for which the msgid's is_format flag
-       is one of 'yes' or 'possible'.  We don't check msgids with is_format
-       'no' or 'impossible', to obey the programmer's order.  We don't check
-       msgids with is_format 'undecided' because that would introduce too
-       many checks, thus forcing the programmer to add "xgettext: no-c-format"
-       anywhere where a translator wishes to use a percent sign.  */
-    for (i = 0; i < NFORMATS; i++)
-      if (possible_format_p (is_format[i]))
-	{
-	  /* At runtime, we can assume the program passes arguments that
-	     fit well for msgid.  We must signal an error if msgstr wants
-	     more arguments that msgid accepts.
-	     If msgstr wants fewer arguments than msgid, it wouldn't lead
-	     to a crash at runtime, but we nevertheless give an error because
-	     1) this situation occurs typically after the programmer has
-		added some arguments to msgid, so we must make the translator
-		specially aware of it (more than just "fuzzy"),
-	     2) it is generally wrong if a translation wants to ignore
-		arguments that are used by other translations.  */
-
-	  struct formatstring_parser *parser = formatstring_parsers[i];
-	  char *invalid_reason = NULL;
-	  void *msgid_descr =
-	    parser->parse (msgid_plural != NULL ? msgid_plural : msgid,
-			   &invalid_reason);
-
-	  if (msgid_descr != NULL)
-	    {
-	      char buf[18+1];
-	      const char *pretty_msgstr = "msgstr";
-	      const char *p_end = msgstr + msgstr_len;
-	      const char *p;
-
-	      for (p = msgstr, j = 0; p < p_end; p += strlen (p) + 1, j++)
-		{
-		  void *msgstr_descr;
-
-		  if (msgid_plural != NULL)
-		    {
-		      sprintf (buf, "msgstr[%u]", j);
-		      pretty_msgstr = buf;
-		    }
-
-		  msgstr_descr = parser->parse (p, &invalid_reason);
-
-		  if (msgstr_descr != NULL)
-		    {
-		      if (parser->check (msgid_pos, msgid_descr, msgstr_descr,
-					 msgid_plural == NULL,
-					 true, pretty_msgstr))
-			exit_status = EXIT_FAILURE;
-
-		      parser->free (msgstr_descr);
-		    }
-		  else
-		    {
-		      error_with_progname = false;
-		      error_at_line (0, 0, msgid_pos->file_name,
-				     msgid_pos->line_number,
-				     _("\
-'%s' is not a valid %s format string, unlike 'msgid'. Reason: %s"),
-				     pretty_msgstr, format_language_pretty[i],
-				     invalid_reason);
-		      error_with_progname = true;
-		      exit_status = EXIT_FAILURE;
-		      free (invalid_reason);
-		    }
-		}
-
-	      parser->free (msgid_descr);
-	    }
-	  else
-	    free (invalid_reason);
-	}
+       of format specifications.  */
+    {
+      curr_msgid_pos = *msgid_pos;
+      if (check_msgid_msgstr_format (msgid, msgid_plural, msgstr, msgstr_len,
+				     is_format, formatstring_error_logger))
+	exit_status = EXIT_FAILURE;
+    }
 
   if (check_accelerators && msgid_plural == NULL)
     /* Test 4: Check that if msgid is a menu item with a keyboard accelerator,
@@ -1413,7 +1457,8 @@ msgfmt_set_domain (default_po_reader_ty *this, char *name)
 {
   /* If no output file was given, we change it with each `domain'
      directive.  */
-  if (!java_mode && !tcl_mode && !qt_mode && output_file_name == NULL)
+  if (!java_mode && !csharp_mode && !csharp_resources_mode && !tcl_mode
+      && !qt_mode && output_file_name == NULL)
     {
       size_t correct;
 
