@@ -1,5 +1,5 @@
 /* Message list concatenation and duplicate handling.
-   Copyright (C) 2001-2002 Free Software Foundation, Inc.
+   Copyright (C) 2001-2003 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software; you can redistribute it and/or modify
@@ -19,29 +19,41 @@
 #ifndef _MSGL_CAT_H
 #define _MSGL_CAT_H
 
+#include <stdbool.h>
+
 #include "message.h"
 #include "str-list.h"
 
-#include <stdbool.h>
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 
 /* These variables control which messages are selected.  */
-extern int more_than;
-extern int less_than;
+extern DLL_VARIABLE int more_than;
+extern DLL_VARIABLE int less_than;
 
 /* If true, use the first available translation.
    If false, merge all available translations into one and fuzzy it.  */
-extern bool use_first;
+extern DLL_VARIABLE bool use_first;
 
 /* If true, merge like msgcomm.
    If false, merge like msgcat and msguniq.  */
-extern bool msgcomm_mode;
+extern DLL_VARIABLE bool msgcomm_mode;
 
 /* If true, omit the header entry.
    If false, keep the header entry present in the input.  */
-extern bool omit_header;
+extern DLL_VARIABLE bool omit_header;
 
 extern msgdomain_list_ty *
        catenate_msgdomain_list (string_list_ty *file_list,
 				const char *to_code);
 
+
+#ifdef __cplusplus
+}
+#endif
+
+
 #endif /* _MSGL_CAT_H */
