@@ -1,5 +1,5 @@
 /* xgettext C/C++/ObjectiveC backend.
-   Copyright (C) 1995-1998, 2000-2003 Free Software Foundation, Inc.
+   Copyright (C) 1995-1998, 2000-2004 Free Software Foundation, Inc.
 
    This file was written by Peter Miller <millerp@canb.auug.org.au>
 
@@ -39,6 +39,8 @@
 
 #define _(s) gettext(s)
 
+#define SIZEOF(a) (sizeof(a) / sizeof(a[0]))
+
 
 /* The ANSI C standard defines several phases of translation:
 
@@ -369,7 +371,7 @@ phase0_getc ()
 }
 
 
-/* Only one pushback character supported, and not '\n'.  */
+/* Supports only one pushback character, and not '\n'.  */
 static inline void
 phase0_ungetc (int c)
 {
@@ -378,12 +380,9 @@ phase0_ungetc (int c)
 }
 
 
-/* 1. line_number handling.  Combine backslash-newline to nothing.
-   Cope with potentially 2 characters of pushback, not just the one that
-   ungetc can cope with.  */
+/* 1. line_number handling.  Combine backslash-newline to nothing.  */
 
-/* Maximum used guaranteed to be < 4.  */
-static unsigned char phase1_pushback[4];
+static unsigned char phase1_pushback[2];
 static int phase1_pushback_length;
 
 
@@ -409,7 +408,7 @@ phase1_getc ()
 	  return '\n';
 
 	case '\\':
-	  c = phase0_getc (fp);
+	  c = phase0_getc ();
 	  if (c != '\n')
 	    {
 	      phase0_ungetc (c);
@@ -425,6 +424,7 @@ phase1_getc ()
 }
 
 
+/* Supports 2 characters of pushback.  */
 static void
 phase1_ungetc (int c)
 {
@@ -438,6 +438,8 @@ phase1_ungetc (int c)
       /* FALLTHROUGH */
 
     default:
+      if (phase1_pushback_length == SIZEOF (phase1_pushback))
+	abort ();
       phase1_pushback[phase1_pushback_length++] = c;
       break;
     }
@@ -448,8 +450,7 @@ phase1_ungetc (int c)
    sane human beings vomit copiously at the mention of trigraphs, which
    is why they are an option.  */
 
-/* Maximum used guaranteed to be < 4.  */
-static unsigned char phase2_pushback[4];
+static unsigned char phase2_pushback[1];
 static int phase2_pushback_length;
 
 
@@ -500,11 +501,16 @@ phase2_getc ()
 }
 
 
+/* Supports only one pushback character.  */
 static void
 phase2_ungetc (int c)
 {
   if (c != EOF)
-    phase2_pushback[phase2_pushback_length++] = c;
+    {
+      if (phase2_pushback_length == SIZEOF (phase2_pushback))
+	abort ();
+      phase2_pushback[phase2_pushback_length++] = c;
+    }
 }
 
 
@@ -512,8 +518,7 @@ phase2_ungetc (int c)
    line.  Basically, all you need to do is elide "\\\n" sequences from
    the input.  */
 
-/* Maximum used guaranteed to be < 4.  */
-static unsigned char phase3_pushback[4];
+static unsigned char phase3_pushback[2];
 static int phase3_pushback_length;
 
 
@@ -537,11 +542,16 @@ phase3_getc ()
 }
 
 
+/* Supports 2 characters of pushback.  */
 static void
 phase3_ungetc (int c)
 {
   if (c != EOF)
-    phase3_pushback[phase3_pushback_length++] = c;
+    {
+      if (phase3_pushback_length == SIZEOF (phase3_pushback))
+	abort ();
+      phase3_pushback[phase3_pushback_length++] = c;
+    }
 }
 
 
@@ -581,7 +591,7 @@ comment_line_end (size_t chars_to_remove)
       buffer = xrealloc (buffer, bufmax);
     }
   buffer[buflen] = '\0';
-  xgettext_comment_add (buffer);
+  savable_comment_add (buffer);
 }
 
 
@@ -673,6 +683,7 @@ phase4_getc ()
 }
 
 
+/* Supports only one pushback character.  */
 static void
 phase4_ungetc (int c)
 {
@@ -710,6 +721,8 @@ struct token_ty
 {
   token_type_ty type;
   char *string;		/* for token_type_name, token_type_string_literal */
+  refcounted_string_list_ty *comment;	/* for token_type_string_literal,
+					   token_type_objc_special */
   long number;
   int line_number;
 };
@@ -872,14 +885,16 @@ free_token (token_ty *tp)
 {
   if (tp->type == token_type_name || tp->type == token_type_string_literal)
     free (tp->string);
+  if (tp->type == token_type_string_literal
+      || tp->type == token_type_objc_special)
+    drop_reference (tp->comment);
 }
 
 
 /* 5. Parse each resulting logical line as preprocessing tokens and
    white space.  Preprocessing tokens and C tokens don't always match.  */
 
-/* Maximum used guaranteed to be < 4.  */
-static token_ty phase5_pushback[4];
+static token_ty phase5_pushback[1];
 static int phase5_pushback_length;
 
 
@@ -1124,6 +1139,7 @@ phase5_get (token_ty *tp)
       buffer[bufpos] = 0;
       tp->type = token_type_string_literal;
       tp->string = xstrdup (buffer);
+      tp->comment = add_reference (savable_comment);
       return;
 
     case '(':
@@ -1150,6 +1166,7 @@ phase5_get (token_ty *tp)
       if (objc_extensions)
 	{
 	  tp->type = token_type_objc_special;
+	  tp->comment = add_reference (savable_comment);
 	  return;
 	}
       /* FALLTHROUGH */
@@ -1164,11 +1181,16 @@ phase5_get (token_ty *tp)
 }
 
 
+/* Supports only one pushback token.  */
 static void
 phase5_unget (token_ty *tp)
 {
   if (tp->type != token_type_eof)
-    phase5_pushback[phase5_pushback_length++] = *tp;
+    {
+      if (phase5_pushback_length == SIZEOF (phase5_pushback))
+	abort ();
+      phase5_pushback[phase5_pushback_length++] = *tp;
+    }
 }
 
 
@@ -1219,8 +1241,7 @@ phaseX_get (token_ty *tp)
    we care about are the #line and #define directive.  We throw all the
    others away.  */
 
-/* Maximum used guaranteed to be < 4.  */
-static token_ty phase6_pushback[4];
+static token_ty phase6_pushback[2];
 static int phase6_pushback_length;
 
 
@@ -1298,16 +1319,21 @@ phase6_get (token_ty *tp)
 	free_token (&buf[j]);
 
       /* We must reset the selected comments.  */
-      xgettext_comment_reset ();
+      savable_comment_reset ();
     }
 }
 
 
+/* Supports 2 tokens of pushback.  */
 static void
 phase6_unget (token_ty *tp)
 {
   if (tp->type != token_type_eof)
-    phase6_pushback[phase6_pushback_length++] = *tp;
+    {
+      if (phase6_pushback_length == SIZEOF (phase6_pushback))
+	abort ();
+      phase6_pushback[phase6_pushback_length++] = *tp;
+    }
 }
 
 
@@ -1368,10 +1394,12 @@ phase8a_get (token_ty *tp)
       new_string[len + 2] = '\0';
       free (tp->string);
       tp->string = new_string;
+      tp->comment = add_reference (savable_comment);
       tp->type = token_type_string_literal;
     }
 }
 
+/* Supports 2 tokens of pushback.  */
 static inline void
 phase8a_unget (token_ty *tp)
 {
@@ -1401,13 +1429,14 @@ phase8b_get (token_ty *tp)
 	     with non-white space tokens.  */
 	  ++newline_count;
 	  if (last_non_comment_line > last_comment_line)
-	    xgettext_comment_reset ();
+	    savable_comment_reset ();
 	  continue;
 	}
       break;
     }
 }
 
+/* Supports 2 tokens of pushback.  */
 static inline void
 phase8b_unget (token_ty *tp)
 {
@@ -1432,9 +1461,12 @@ phase8c_get (token_ty *tp)
       return;
     }
   /* Drop the '@' token and return immediately the following string.  */
+  drop_reference (tmp.comment);
+  tmp.comment = tp->comment;
   *tp = tmp;
 }
 
+/* Supports only one pushback token.  */
 static inline void
 phase8c_unget (token_ty *tp)
 {
@@ -1501,6 +1533,9 @@ struct xgettext_token_ty
      xgettext_token_type_keyword, xgettext_token_type_symbol.  */
   char *string;
 
+  /* This field is used only for xgettext_token_type_string_literal.  */
+  refcounted_string_list_ty *comment;
+
   /* These fields are only for
        xgettext_token_type_keyword,
        xgettext_token_type_string_literal.  */
@@ -1573,10 +1608,15 @@ x_c_lex (xgettext_token_ty *tp)
 
 	  tp->type = xgettext_token_type_string_literal;
 	  tp->string = token.string;
+	  tp->comment = token.comment;
 	  tp->pos.file_name = logical_file_name;
 	  tp->pos.line_number = token.line_number;
 	  return;
 
+	case token_type_objc_special:
+	  drop_reference (token.comment);
+	  /* FALLTHROUGH */
+
 	default:
 	  last_non_comment_line = newline_count;
 
@@ -1739,7 +1779,11 @@ extract_parenthesized (message_list_ty *mlp,
 
 	case xgettext_token_type_string_literal:
 	  if (extract_all)
-	    remember_a_message (mlp, token.string, inner_context, &token.pos);
+	    {
+	      savable_comment_to_xgettext_comment (token.comment);
+	      remember_a_message (mlp, token.string, inner_context, &token.pos);
+	      savable_comment_reset ();
+	    }
 	  else
 	    {
 	      if (commas_to_skip == 0)
@@ -1747,9 +1791,12 @@ extract_parenthesized (message_list_ty *mlp,
 		  if (plural_mp == NULL)
 		    {
 		      /* Seen an msgid.  */
-		      message_ty *mp =
-			remember_a_message (mlp, token.string,
-					    inner_context, &token.pos);
+		      message_ty *mp;
+
+		      savable_comment_to_xgettext_comment (token.comment);
+		      mp = remember_a_message (mlp, token.string,
+					       inner_context, &token.pos);
+		      savable_comment_reset ();
 		      if (plural_commas > 0)
 			plural_mp = mp;
 		    }
@@ -1764,6 +1811,7 @@ extract_parenthesized (message_list_ty *mlp,
 	      else
 		free (token.string);
 	    }
+	  drop_reference (token.comment);
 	  next_context_iter = null_context_list_iterator;
 	  selectorcall_context_iter = null_context_list_iterator;
 	  state = 0;
