@@ -40,6 +40,8 @@
 
 #define _(s) gettext(s)
 
+#define SIZEOF(a) (sizeof(a) / sizeof(a[0]))
+
 
 /* The Java syntax is defined in the
      Java Language Specification, Second Edition,
@@ -178,6 +180,7 @@ error while reading \"%s\""), real_file_name);
   return c;
 }
 
+/* Supports any number of 'u' and up to 4 arbitrary characters of pushback.  */
 static void
 phase1_ungetc (int c)
 {
@@ -189,18 +192,25 @@ phase1_ungetc (int c)
 	      && phase1_pushback[phase1_pushback_length - 1] >= MULTIPLE_U (0))
 	    phase1_pushback[phase1_pushback_length - 1]++;
 	  else
-	    phase1_pushback[phase1_pushback_length++] = MULTIPLE_U (1);
+	    {
+	      if (phase1_pushback_length == SIZEOF (phase1_pushback))
+		abort ();
+	      phase1_pushback[phase1_pushback_length++] = MULTIPLE_U (1);
+	    }
 	}
       else
-	phase1_pushback[phase1_pushback_length++] = c;
+	{
+	  if (phase1_pushback_length == SIZEOF (phase1_pushback))
+	    abort ();
+	  phase1_pushback[phase1_pushback_length++] = c;
+	}
     }
 }
 
 
 /* Fetch the next single-byte character or Unicode character from the file.
    (Here, as in the Java Language Specification, when we say "Unicode
-   character", we actually mean "UTF-16 encoding unit".)
-   Cope with potentially 2 pushback characters.  */
+   character", we actually mean "UTF-16 encoding unit".)  */
 
 /* Return value of phase 2, 3, 4 when EOF is reached.  */
 #define P2_EOF 0xffff
@@ -221,8 +231,7 @@ phase1_ungetc (int c)
    (RED (c) == 'x')  is equivalent to  (c == 'x' || c == UNICODE ('x')).  */
 #define RED(p2_result) ((p2_result) & 0xffff)
 
-/* Maximum used guaranteed to be < 2.  */
-static int phase2_pushback[2];
+static int phase2_pushback[1];
 static int phase2_pushback_length;
 
 static int
@@ -286,20 +295,23 @@ phase2_getc ()
   return c;
 }
 
+/* Supports only one pushback character.  */
 static void
 phase2_ungetc (int c)
 {
   if (c != P2_EOF)
-    phase2_pushback[phase2_pushback_length++] = c;
+    {
+      if (phase2_pushback_length == SIZEOF (phase2_pushback))
+	abort ();
+      phase2_pushback[phase2_pushback_length++] = c;
+    }
 }
 
 
 /* Fetch the next single-byte character or Unicode character from the file.
    With line number handling.
-   Convert line terminators to '\n' or UNICODE ('\n').
-   Cope with potentially 2 pushback characters.  */
+   Convert line terminators to '\n' or UNICODE ('\n').  */
 
-/* Maximum used guaranteed to be < 2.  */
 static int phase3_pushback[2];
 static int phase3_pushback_length;
 
@@ -350,6 +362,7 @@ phase3_getc ()
   return c;
 }
 
+/* Supports 2 characters of pushback.  */
 static void
 phase3_ungetc (int c)
 {
@@ -357,6 +370,8 @@ phase3_ungetc (int c)
     {
       if (c == '\n')
 	--line_number;
+      if (phase3_pushback_length == SIZEOF (phase3_pushback))
+	abort ();
       phase3_pushback[phase3_pushback_length++] = c;
     }
 }
@@ -557,99 +572,6 @@ free_string_buffer (struct string_buffer *bp)
 /* ======================== Accumulating comments.  ======================== */
 
 
-/* In this backend we cannot use the xgettext_comment* functions directly,
-   because in multiline string expressions like
-           "string1" +
-           "string2"
-   the newline between "string1" and "string2" would cause a call to
-   xgettext_comment_reset(), thus destroying the accumulated comments
-   that we need a little later, when we have concatenated the two strings
-   and pass them to remember_a_message().
-   Instead, we do the bookkeeping of the accumulated comments directly,
-   and save a pointer to the accumulated comments when we read "string1".
-   In order to avoid excessive copying of strings, we use reference
-   counting.  */
-
-typedef struct refcounted_string_list_ty refcounted_string_list_ty;
-struct refcounted_string_list_ty
-{
-  unsigned int refcount;
-  struct string_list_ty contents;
-};
-
-static refcounted_string_list_ty *comment;
-
-static inline refcounted_string_list_ty *
-add_reference (refcounted_string_list_ty *rslp)
-{
-  if (rslp != NULL)
-    rslp->refcount++;
-  return rslp;
-}
-
-static inline void
-drop_reference (refcounted_string_list_ty *rslp)
-{
-  if (rslp != NULL)
-    {
-      if (rslp->refcount > 1)
-	rslp->refcount--;
-      else
-	{
-	  string_list_destroy (&rslp->contents);
-	  free (rslp);
-	}
-    }
-}
-
-static void
-x_java_comment_add (const char *str)
-{
-  if (comment == NULL)
-    {
-      comment = (refcounted_string_list_ty *) xmalloc (sizeof (*comment));
-      comment->refcount = 1;
-      string_list_init (&comment->contents);
-    }
-  else if (comment->refcount > 1)
-    {
-      /* Unshare the list by making copies.  */
-      struct string_list_ty *oldcontents;
-      size_t i;
-
-      comment->refcount--;
-      oldcontents = &comment->contents;
-
-      comment = (refcounted_string_list_ty *) xmalloc (sizeof (*comment));
-      comment->refcount = 1;
-      string_list_init (&comment->contents);
-      for (i = 0; i < oldcontents->nitems; i++)
-	string_list_append (&comment->contents, oldcontents->item[i]);
-    }
-  string_list_append (&comment->contents, str);
-}
-
-static void
-x_java_comment_reset ()
-{
-  drop_reference (comment);
-  comment = NULL;
-}
-
-static void
-x_java_comment_to_xgettext_comment (refcounted_string_list_ty *rslp)
-{
-  xgettext_comment_reset ();
-  if (rslp != NULL)
-    {
-      size_t i;
-
-      for (i = 0; i < rslp->contents.nitems; i++)
-	xgettext_comment_add (rslp->contents.item[i]);
-    }
-}
-
-
 /* Accumulating a single comment line.  */
 
 static struct string_buffer comment_buffer;
@@ -686,7 +608,7 @@ comment_line_end (size_t chars_to_remove)
 	 && (buffer[buflen - 1] == ' ' || buffer[buflen - 1] == '\t'))
     --buflen;
   buffer[buflen] = '\0';
-  x_java_comment_add (buffer);
+  savable_comment_add (buffer);
 }
 
 
@@ -777,6 +699,7 @@ phase4_getc ()
     }
 }
 
+/* Supports only one pushback character.  */
 static void
 phase4_ungetc (int c)
 {
@@ -897,11 +820,15 @@ accumulate_escaped (struct string_buffer *literal, int delimiter)
 	break;
       if (RED (c) == '\n')
 	{
-	  error_with_progname = false;
-	  error (0, 0, _("%s:%d: warning: unterminated character constant"),
-		 logical_file_name, line_number - 1);
-	  error_with_progname = true;
 	  phase3_ungetc (c);
+	  error_with_progname = false;
+	  if (delimiter == '\'')
+	    error (0, 0, _("%s:%d: warning: unterminated character constant"),
+		   logical_file_name, line_number);
+	  else
+	    error (0, 0, _("%s:%d: warning: unterminated string constant"),
+		   logical_file_name, line_number);
+	  error_with_progname = true;
 	  break;
 	}
       if (RED (c) == '\\')
@@ -913,8 +840,7 @@ accumulate_escaped (struct string_buffer *literal, int delimiter)
 
 /* Combine characters into tokens.  Discard whitespace.  */
 
-/* Maximum used guaranteed to be < 4.  */
-static token_ty phase5_pushback[4];
+static token_ty phase5_pushback[3];
 static int phase5_pushback_length;
 
 static void
@@ -944,7 +870,7 @@ phase5_get (token_ty *tp)
 	{
 	case '\n':
 	  if (last_non_comment_line > last_comment_line)
-	    x_java_comment_reset ();
+	    savable_comment_reset ();
 	  /* FALLTHROUGH */
 	case ' ':
 	case '\t':
@@ -1077,7 +1003,7 @@ phase5_get (token_ty *tp)
 	    accumulate_escaped (&literal, '"');
 	    tp->string = xstrdup (string_buffer_result (&literal));
 	    free_string_buffer (&literal);
-	    tp->comment = add_reference (comment);
+	    tp->comment = add_reference (savable_comment);
 	    tp->type = token_type_string_literal;
 	    return;
 	  }
@@ -1118,11 +1044,16 @@ phase5_get (token_ty *tp)
     }
 }
 
+/* Supports 3 tokens of pushback.  */
 static void
 phase5_unget (token_ty *tp)
 {
   if (tp->type != token_type_eof)
-    phase5_pushback[phase5_pushback_length++] = *tp;
+    {
+      if (phase5_pushback_length == SIZEOF (phase5_pushback))
+	abort ();
+      phase5_pushback[phase5_pushback_length++] = *tp;
+    }
 }
 
 
@@ -1133,8 +1064,7 @@ phase5_unget (token_ty *tp)
      - the token after this expression is not '.' (because then the last
        string could be part of a method call expression).  */
 
-/* Maximum used guaranteed to be < 4.  */
-static token_ty phase6_pushback[4];
+static token_ty phase6_pushback[2];
 static int phase6_pushback_length;
 
 static token_type_ty phase6_last;
@@ -1195,11 +1125,16 @@ phase6_get (token_ty *tp)
   phase6_last = tp->type;
 }
 
+/* Supports 2 tokens of pushback.  */
 static void
 phase6_unget (token_ty *tp)
 {
   if (tp->type != token_type_eof)
-    phase6_pushback[phase6_pushback_length++] = *tp;
+    {
+      if (phase6_pushback_length == SIZEOF (phase6_pushback))
+	abort ();
+      phase6_pushback[phase6_pushback_length++] = *tp;
+    }
 }
 
 
@@ -1209,6 +1144,7 @@ x_java_lex (token_ty *tp)
   phase6_get (tp);
 }
 
+/* Supports 2 tokens of pushback.  */
 static void
 x_java_unlex (token_ty *tp)
 {
@@ -1447,9 +1383,9 @@ extract_parenthesized (message_list_ty *mlp, token_type_ty terminator,
 	    if (extract_all)
 	      {
 		xgettext_current_source_encoding = po_charset_utf8;
-		x_java_comment_to_xgettext_comment (token.comment);
+		savable_comment_to_xgettext_comment (token.comment);
 		remember_a_message (mlp, token.string, inner_context, &pos);
-		x_java_comment_reset ();
+		savable_comment_reset ();
 		xgettext_current_source_encoding = xgettext_global_source_encoding;
 	      }
 	    else
@@ -1462,10 +1398,10 @@ extract_parenthesized (message_list_ty *mlp, token_type_ty terminator,
 			message_ty *mp;
 
 			xgettext_current_source_encoding = po_charset_utf8;
-			x_java_comment_to_xgettext_comment (token.comment);
+			savable_comment_to_xgettext_comment (token.comment);
 			mp = remember_a_message (mlp, token.string,
 						 inner_context, &pos);
-			x_java_comment_reset ();
+			savable_comment_reset ();
 			xgettext_current_source_encoding = xgettext_global_source_encoding;
 			if (plural_commas > 0)
 			  plural_mp = mp;
