@@ -1,5 +1,5 @@
 /* xgettext Python backend.
-   Copyright (C) 2002 Free Software Foundation, Inc.
+   Copyright (C) 2002-2003 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2002.
 
    This program is free software; you can redistribute it and/or modify
@@ -21,12 +21,16 @@
   { "py",        "Python"   },						\
 
 #define SCANNERS_PYTHON \
-  { "Python",     extract_python, &formatstring_python },		\
+  { "Python",		extract_python,					  \
+			&flag_table_python, &formatstring_python, NULL }, \
 
 /* Scan a Python file and add its translatable strings to mdlp.  */
 extern void extract_python (FILE *fp, const char *real_filename,
 			    const char *logical_filename,
+			    flag_context_list_table_ty *flag_table,
 			    msgdomain_list_ty *mdlp);
 
 extern void x_python_keyword (const char *keyword);
 extern void x_python_extract_all (void);
+
+extern void init_flag_table_python (void);
