@@ -1,5 +1,5 @@
 /* xgettext Tcl backend.
-   Copyright (C) 2002-2003 Free Software Foundation, Inc.
+   Copyright (C) 2002-2003, 2005 Free Software Foundation, Inc.
 
    This file was written by Bruno Haible <haible@clisp.cons.org>, 2002.
 
@@ -43,6 +43,8 @@
 
 #define _(s) gettext(s)
 
+#define SIZEOF(a) (sizeof(a) / sizeof(a[0]))
+
 
 /* The Tcl syntax is defined in the Tcl.n manual page.
    Summary of Tcl syntax:
@@ -167,13 +169,12 @@ do_ungetc (int c)
 
 
 /* Combine backslash followed by newline and additional whitespace to
-   a single space.  Cope with potentially 2 characters of pushback.  */
+   a single space.  */
 
 /* An int that becomes a space when casted to 'unsigned char'.  */
 #define BS_NL (UCHAR_MAX + 1 + ' ')
 
-/* Maximum used guaranteed to be < 4.  */
-static int phase1_pushback[4];
+static int phase1_pushback[1];
 static int phase1_pushback_length;
 
 static int
@@ -209,6 +210,7 @@ phase1_getc ()
   return BS_NL;
 }
 
+/* Supports only one pushback character.  */
 static void
 phase1_ungetc (int c)
 {
@@ -223,6 +225,8 @@ phase1_ungetc (int c)
       /* FALLTHROUGH */
 
     default:
+      if (phase1_pushback_length == SIZEOF (phase1_pushback))
+	abort ();
       phase1_pushback[phase1_pushback_length++] = c;
       break;
     }
@@ -239,8 +243,7 @@ phase1_ungetc (int c)
 /* An int that becomes a closing brace when casted to 'unsigned char'.  */
 #define CL_BRACE (UCHAR_MAX + 1 + '}')
 
-/* Maximum used guaranteed to be < 4.  */
-static int phase2_pushback[4];
+static int phase2_pushback[2];
 static int phase2_pushback_length;
 
 /* Brace nesting depth inside the current character group.  */
@@ -287,6 +290,7 @@ phase2_getc ()
   return c;
 }
 
+/* Supports 2 characters of pushback.  */
 static void
 phase2_ungetc (int c)
 {
@@ -307,6 +311,8 @@ phase2_ungetc (int c)
 	  ++brace_depth;
 	  break;
 	}
+      if (phase2_pushback_length == SIZEOF (phase2_pushback))
+	abort ();
       phase2_pushback[phase2_pushback_length++] = c;
     }
 }
@@ -570,7 +576,8 @@ static enum word_type read_command_list (int looking_for,
 					 flag_context_ty outer_context);
 
 /* Accumulate tokens into the given word.
-   'looking_for' denotes a parse terminator combination.  */
+   'looking_for' denotes a parse terminator combination.
+   Return the first character past the token.  */
 static int
 accumulate_word (struct word *wp, enum terminator looking_for,
 		 flag_context_ty context)
@@ -712,7 +719,8 @@ read_word (struct word *wp, int looking_for, flag_context_ty context)
 
   do
     c = phase2_getc ();
-  while (c == ' ' || c == BS_NL);
+  while (c == ' ' || c == BS_NL
+	 || c == '\t' || c == '\v' || c == '\f' || c == '\r');
 
   if (c == EOF)
     {
